@@ -1,7 +1,6 @@
 /* decrypt-data.c - Decrypt an encrypted data packet
  * Copyright (C) 1998-2001, 2005-2006, 2009 Free Software Foundation, Inc.
  * Copyright (C) 1998-2001, 2005-2006, 2009, 2018 Werner Koch
- * Copyright (C) 2020 g10 Code GmbH
  *
  * This file is part of GnuPG.
  *
@@ -44,7 +43,7 @@ static int decode_filter ( void *opaque, int control, IOBUF a,
 /* Our context object.  */
 struct decode_filter_context_s
 {
-  /* Recounter (max value is 2).  We need it because we do not know
+  /* Redcounter (max value is 2).  We need it because we do not know
    * whether the iobuf or the outer control code frees this object
    * first.  */
   int  refcount;
@@ -73,6 +72,9 @@ struct decode_filter_context_s
    *   3 = premature EOF (general)       */
   unsigned int eof_seen : 2;
 
+  /* Flag to convey an error from aead_checktag.  */
+  unsigned int checktag_failed : 1;
+
   /* The actually used cipher algo for AEAD.  */
   byte cipher_algo;
 
@@ -206,6 +208,8 @@ aead_checktag (decode_filter_ctx_t dfx, int final, const void *tagbuf)
     {
       log_error ("gcry_cipher_checktag%s failed: %s\n",
                  final? " (final)":"", gpg_strerror (err));
+      write_status_error ("aead_checktag", err);
+      dfx->checktag_failed = 1;
       return err;
     }
   if (DBG_FILTER)
@@ -230,8 +234,8 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
   byte *p;
   int rc=0, c, i;
   byte temp[32];
-  unsigned blocksize;
-  unsigned nprefix;
+  unsigned int blocksize;
+  unsigned int nprefix;
 
   *compliance_error = 0;
 
@@ -263,11 +267,13 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
   /* Check compliance.  */
   if (!gnupg_cipher_is_allowed (opt.compliance, 0, dek->algo, ciphermode))
     {
-      log_error (_("cipher algorithm '%s' may not be used in %s mode\n"),
+      gpgrt_log (opt.show_only_session_key? GPGRT_LOGLVL_INFO
+                 /*                     */: GPGRT_LOGLVL_ERROR,
+                 _("cipher algorithm '%s' may not be used in %s mode\n"),
 		 openpgp_cipher_algo_mode_name (dek->algo,ed->aead_algo),
 		 gnupg_compliance_option_string (opt.compliance));
       *compliance_error = 1;
-      if (opt.flags.require_compliance)
+      if (opt.flags.require_compliance && !opt.show_only_session_key)
         {
           /* We fail early in this case because it does not make sense
            * to first decrypt everything.  */
@@ -276,15 +282,19 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
         }
     }
 
-  write_status_printf (STATUS_DECRYPTION_INFO, "%d %d %d",
-                       ed->mdc_method, dek->algo, 0);
+  write_status_printf (STATUS_DECRYPTION_INFO, "%d %d %d %d",
+                       ed->mdc_method, dek->algo, ed->aead_algo,
+                       *compliance_error);
 
   if (opt.show_session_key)
     {
-      char numbuf[25];
+      char numbuf[30];
       char *hexbuf;
 
-      snprintf (numbuf, sizeof numbuf, "%d:", dek->algo);
+      if (ed->aead_algo)
+        snprintf (numbuf, sizeof numbuf, "%d.%u:", dek->algo, ed->aead_algo);
+      else
+        snprintf (numbuf, sizeof numbuf, "%d:", dek->algo);
       hexbuf = bin2hex (dek->key, dek->keylen, NULL);
       if (!hexbuf)
         {
@@ -294,6 +304,11 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
       log_info ("session key: '%s%s'\n", numbuf, hexbuf);
       write_status_strings (STATUS_SESSION_KEY, numbuf, hexbuf, NULL);
       xfree (hexbuf);
+      if (opt.show_only_session_key)
+        {
+          rc = 0;
+          goto leave;
+        }
     }
 
   rc = openpgp_cipher_test_algo (dek->algo);
@@ -421,11 +436,11 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
         {
           log_info (_("WARNING: message was encrypted with"
                       " a weak key in the symmetric cipher.\n"));
-          rc = 0;
+          rc=0;
         }
       else if (rc)
         {
-          log_error("key setup failed: %s\n", gpg_strerror (rc) );
+          log_error ("key setup failed: %s\n", gpg_strerror (rc) );
           goto leave;
         }
 
@@ -475,6 +490,7 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
   dfx->refcount++;
   dfx->partial = !!ed->is_partial;
   dfx->length = ed->len;
+  dfx->checktag_failed = 0;
   if (ed->aead_algo)
     iobuf_push_filter ( ed->buf, aead_decode_filter, dfx );
   else if (ed->mdc_method)
@@ -486,12 +502,15 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
     {
       char *filename = NULL;
       estream_t fp;
+
       rc = get_output_file ("", 0, ed->buf, &filename, &fp);
       if (! rc)
         {
-          iobuf_t output = iobuf_esopen (fp, "w", 0);
+          iobuf_t output = iobuf_esopen (fp, "w", 0, 0);
           armor_filter_context_t *afx = NULL;
 
+	  es_setbuf (fp, NULL);
+
           if (opt.armor)
             {
               afx = new_armor_context ();
@@ -507,8 +526,7 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
                        filename, gpg_strerror (rc));
 
           iobuf_close (output);
-          if (afx)
-            release_armor_context (afx);
+          release_armor_context (afx);
         }
       xfree (filename);
     }
@@ -518,6 +536,10 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
   ed->buf = NULL;
   if (dfx->eof_seen > 1 )
     rc = gpg_error (GPG_ERR_INV_PACKET);
+  else if (dfx->checktag_failed)
+    {
+      rc = gpg_error (GPG_ERR_BAD_SIGNATURE);
+    }
   else if ( ed->mdc_method )
     {
       /* We used to let parse-packet.c handle the MDC packet but this
@@ -546,8 +568,8 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
           || datalen != 20
           || memcmp (gcry_md_read (dfx->mdc_hash, 0), dfx->holdback+2, datalen))
         rc = gpg_error (GPG_ERR_BAD_SIGNATURE);
-      /* log_printhex(dfx->holdback, 22, "MDC message:"); */
-      /* log_printhex(gcry_md_read (dfx->mdc_hash,0), datalen, "MDC calc:"); */
+      /* log_printhex("MDC message:", dfx->holdback, 22); */
+      /* log_printhex("MDC calc:", gcry_md_read (dfx->mdc_hash,0), datalen); */
     }
 
  leave:
@@ -616,7 +638,7 @@ static gpg_error_t
 aead_underflow (decode_filter_ctx_t dfx, iobuf_t a, byte *buf, size_t *ret_len)
 {
   const size_t size = *ret_len; /* The allocated size of BUF.  */
-  gpg_error_t err;
+  gpg_error_t err = 0;
   size_t totallen = 0; /* The number of bytes to return on success or EOF.  */
   size_t off = 0;      /* The offset into the buffer.  */
   size_t len;          /* The current number of bytes in BUF+OFF.  */
@@ -671,8 +693,10 @@ aead_underflow (decode_filter_ctx_t dfx, iobuf_t a, byte *buf, size_t *ret_len)
   /* Decrypt the buffer.  This first requires a loop to handle the
    * case when a chunk ends within the buffer.  */
   if (DBG_FILTER)
-    log_debug ("decrypt: chunklen=%ju total=%ju size=%zu len=%zu%s\n",
-               dfx->chunklen, dfx->total, size, len,
+    log_debug ("decrypt: chunklen=%llu total=%llu size=%zu len=%zu%s\n",
+               (unsigned long long)dfx->chunklen,
+               (unsigned long long)dfx->total,
+               size, len,
                dfx->eof_seen? " eof":"");
 
   while (len && dfx->chunklen + len >= dfx->chunksize)
@@ -708,8 +732,8 @@ aead_underflow (decode_filter_ctx_t dfx, iobuf_t a, byte *buf, size_t *ret_len)
       len -= n;
 
       if (DBG_FILTER)
-        log_debug ("ndecrypted: %zu (nchunk=%ju) bytes left: %zu at off=%zu\n",
-                   totallen, dfx->chunklen, len, off);
+        log_debug ("ndecrypted: %zu (nchunk=%llu) bytes left: %zu at off=%zu\n",
+                   totallen, (unsigned long long)dfx->chunklen, len, off);
 
       /* Check the tag.  */
       if (len < 16)
@@ -790,7 +814,8 @@ aead_underflow (decode_filter_ctx_t dfx, iobuf_t a, byte *buf, size_t *ret_len)
       dfx->chunklen += len;
       dfx->total += len;
       if (DBG_FILTER)
-        log_debug ("ndecrypted: %zu (nchunk=%ju)\n", totallen, dfx->chunklen);
+        log_debug ("ndecrypted: %zu (nchunk=%llu)\n",
+                   totallen, (unsigned long long)dfx->chunklen);
     }
 
   if (dfx->eof_seen)
@@ -921,7 +946,7 @@ mdc_decode_filter (void *opaque, int control, IOBUF a,
       n = fill_buffer (dfx, a, buf, 44, 22);
       if (n == 44)
         {
-          /* We have enough stuff - flush the deferred stuff.  */
+          /* We have enough stuff - flush the holdback buffer.  */
           if ( !dfx->holdbacklen )  /* First time. */
             {
               memcpy (buf, buf+22, 22);
@@ -931,6 +956,7 @@ mdc_decode_filter (void *opaque, int control, IOBUF a,
             {
               memcpy (buf, dfx->holdback, 22);
 	    }
+
           /* Fill up the buffer. */
           n = fill_buffer (dfx, a, buf, size, n);
 
@@ -940,7 +966,7 @@ mdc_decode_filter (void *opaque, int control, IOBUF a,
           memcpy (dfx->holdback, buf+n, 22 );
           dfx->holdbacklen = 22;
 	}
-      else if ( !dfx->holdbacklen )  /* EOF seen but empty holdback buffer. */
+      else if ( !dfx->holdbacklen ) /* EOF seen but empty holdback. */
         {
           /* This is bad because it means an incomplete hash. */
           n -= 22;
