@@ -1,6 +1,6 @@
 /* gpgv.c - The GnuPG signature verify utility
  * Copyright (C) 1998-2020 Free Software Foundation, Inc.
- * Copyright (C) 1998-2019 Werner Koch
+ * Copyright (C) 1997-2019 Werner Koch
  * Copyright (C) 2015-2020 g10 Code GmbH
  *
  * This file is part of GnuPG.
@@ -58,6 +58,7 @@ enum cmd_and_opt_values {
   oQuiet	  = 'q',
   oVerbose	  = 'v',
   oOutput	  = 'o',
+  oPrintNotation  = 'N',
   oBatch	  = 500,
   oKeyring,
   oIgnoreTimeConflict,
@@ -68,11 +69,12 @@ enum cmd_and_opt_values {
   oWeakDigest,
   oEnableSpecialFilenames,
   oDebug,
+  oAssertPubkeyAlgo,
   aTest
 };
 
 
-static ARGPARSE_OPTS opts[] = {
+static gpgrt_opt_t opts[] = {
   ARGPARSE_group (300, N_("@\nOptions:\n ")),
 
   ARGPARSE_s_n (oVerbose, "verbose", N_("verbose")),
@@ -90,7 +92,10 @@ static ARGPARSE_OPTS opts[] = {
   ARGPARSE_s_s (oWeakDigest, "weak-digest",
                 N_("|ALGO|reject signatures made with ALGO")),
   ARGPARSE_s_n (oEnableSpecialFilenames, "enable-special-filenames", "@"),
+  ARGPARSE_s_s (oPrintNotation, "print-notation",
+                N_("|NAME|print the notation NAME to stdout")),
   ARGPARSE_s_s (oDebug, "debug", "@"),
+  ARGPARSE_s_s (oAssertPubkeyAlgo,"assert-pubkey-algo", "@"),
 
   ARGPARSE_end ()
 };
@@ -118,7 +123,8 @@ static struct debug_flags_s debug_flags [] =
 
 
 int g10_errors_seen = 0;
-
+int assert_signer_true = 0;
+int assert_pubkey_algo_false = 0;
 
 static char *
 make_libversion (const char *libname, const char *(*getfnc)(const char*))
@@ -172,14 +178,14 @@ my_strusage( int level )
 int
 main( int argc, char **argv )
 {
-  ARGPARSE_ARGS pargs;
+  gpgrt_argparse_t pargs;
   int rc=0;
   strlist_t sl;
   strlist_t nrings = NULL;
   ctrl_t ctrl;
 
   early_system_init ();
-  set_strusage (my_strusage);
+  gpgrt_set_strusage (my_strusage);
   log_set_prefix ("gpgv", GPGRT_LOG_WITH_PREFIX);
 
   /* Make sure that our subsystems are ready.  */
@@ -210,7 +216,7 @@ main( int argc, char **argv )
   pargs.argc = &argc;
   pargs.argv = &argv;
   pargs.flags= ARGPARSE_FLAG_KEEP;
-  while (gnupg_argparser (&pargs, opts, NULL))
+  while (gpgrt_argparser (&pargs, opts, NULL))
     {
       switch (pargs.r_opt)
         {
@@ -251,11 +257,28 @@ main( int argc, char **argv )
         case oEnableSpecialFilenames:
           enable_special_filenames ();
           break;
+
+        case oAssertPubkeyAlgo:
+          if (!opt.assert_pubkey_algos)
+            opt.assert_pubkey_algos = xstrdup (pargs.r.ret_str);
+          else
+            {
+              char *tmp = opt.assert_pubkey_algos;
+              opt.assert_pubkey_algos = xstrconcat (tmp, ",",
+                                                    pargs.r.ret_str, NULL);
+              xfree (tmp);
+            }
+          break;
+
+        case oPrintNotation:
+          append_to_strlist (&opt.print_notations, pargs.r.ret_str);
+          break;
+
         default : pargs.err = ARGPARSE_PRINT_ERROR; break;
 	}
     }
 
-  gnupg_argparse (NULL, &pargs, NULL);  /* Release internal state.  */
+  gpgrt_argparse (NULL, &pargs, NULL);  /* Release internal state.  */
 
   if (log_get_errorcount (0))
     g10_exit(2);
@@ -288,10 +311,18 @@ main( int argc, char **argv )
 
 
 void
-g10_exit( int rc )
+g10_exit (int rc)
 {
-  rc = rc? rc : log_get_errorcount(0)? 2 : g10_errors_seen? 1 : 0;
-  exit(rc );
+  if (rc)
+    ;
+  else if (log_get_errorcount(0))
+    rc = 2;
+  else if (g10_errors_seen)
+    rc = 1;
+  else if (opt.assert_pubkey_algos && assert_pubkey_algo_false)
+    rc = 1;
+
+  exit (rc);
 }
 
 
@@ -299,10 +330,13 @@ g10_exit( int rc )
  * We have to override the trustcheck from pkclist.c because
  * this utility assumes that all keys in the keyring are trustworthy
  */
-int
-check_signatures_trust (ctrl_t ctrl, PKT_signature *sig)
+gpg_error_t
+check_signatures_trust (ctrl_t ctrl, kbnode_t kblock,
+                        PKT_public_key *pk, PKT_signature *sig)
 {
   (void)ctrl;
+  (void)kblock;
+  (void)pk;
   (void)sig;
   return 0;
 }
@@ -399,6 +433,14 @@ get_ownertrust (ctrl_t ctrl, PKT_public_key *pk)
   return TRUST_UNKNOWN;
 }
 
+const char *
+get_ownertrust_string (ctrl_t ctrl, PKT_public_key *pk, int no_create)
+{
+  (void)ctrl;
+  (void)pk;
+  (void)no_create;
+  return "";
+}
 
 /* Stubs:
  * Because we only work with trusted keys, it does not make sense to
@@ -420,17 +462,20 @@ keyserver_any_configured (ctrl_t ctrl)
 }
 
 int
-keyserver_import_keyid (u32 *keyid, void *dummy, unsigned int flags)
+keyserver_import_keyid (ctrl_t ctrl,
+                        u32 *keyid,struct keyserver_spec *keyserver,
+                        unsigned int flags)
 {
+  (void)ctrl;
   (void)keyid;
-  (void)dummy;
+  (void)keyserver;
   (void)flags;
   return -1;
 }
 
 int
-keyserver_import_fprint (ctrl_t ctrl, const byte *fprint,size_t fprint_len,
-			 struct keyserver_spec *keyserver, unsigned int flags)
+keyserver_import_fpr (ctrl_t ctrl, const byte *fprint,size_t fprint_len,
+		      struct keyserver_spec *keyserver, unsigned int flags)
 {
   (void)ctrl;
   (void)fprint;
@@ -441,8 +486,8 @@ keyserver_import_fprint (ctrl_t ctrl, const byte *fprint,size_t fprint_len,
 }
 
 int
-keyserver_import_fprint_ntds (ctrl_t ctrl,
-                              const byte *fprint, size_t fprint_len)
+keyserver_import_fpr_ntds (ctrl_t ctrl,
+                           const byte *fprint, size_t fprint_len)
 {
   (void)ctrl;
   (void)fprint;
@@ -451,17 +496,14 @@ keyserver_import_fprint_ntds (ctrl_t ctrl,
 }
 
 int
-keyserver_import_cert (const char *name)
-{
-  (void)name;
-  return -1;
-}
-
-int
-keyserver_import_pka (const char *name,unsigned char *fpr)
+keyserver_import_cert (ctrl_t ctrl, const char *name, int dane_mode,
+                       unsigned char **fpr,size_t *fpr_len)
 {
+  (void)ctrl;
   (void)name;
+  (void)dane_mode;
   (void)fpr;
+  (void)fpr_len;
   return -1;
 }
 
@@ -477,11 +519,17 @@ keyserver_import_wkd (ctrl_t ctrl, const char *name, unsigned int flags,
   return GPG_ERR_BUG;
 }
 
-int
-keyserver_import_mbox (const char *name,struct keyserver_spec *spec)
+gpg_error_t
+keyserver_import_mbox (ctrl_t ctrl, const char *mbox,
+                       unsigned char **fpr, size_t *fprlen,
+                       struct keyserver_spec *keyserver, unsigned int flags)
 {
-  (void)name;
-  (void)spec;
+  (void)ctrl;
+  (void)mbox;
+  (void)fpr;
+  (void)fprlen;
+  (void)keyserver;
+  (void)flags;
   return -1;
 }
 
@@ -530,7 +578,7 @@ import_included_key_block (ctrl_t ctrl, kbnode_t keyblock)
  * No encryption here but mainproc links to these functions.
  */
 gpg_error_t
-get_session_key (ctrl_t ctrl, PKT_pubkey_enc *k, DEK *dek)
+get_session_key (ctrl_t ctrl, struct seskey_enc_list *k, DEK *dek)
 {
   (void)ctrl;
   (void)k;
@@ -613,14 +661,11 @@ parse_preferred_keyserver(PKT_signature *sig)
   return NULL;
 }
 
-struct keyserver_spec *
-parse_keyserver_uri (const char *uri, int require_scheme,
-                     const char *configname, unsigned int configlineno)
+keyserver_spec_t
+parse_keyserver_uri (const char *string, int require_scheme)
 {
-  (void)uri;
+  (void)string;
   (void)require_scheme;
-  (void)configname;
-  (void)configlineno;
   return NULL;
 }
 
@@ -632,11 +677,14 @@ free_keyserver_spec (struct keyserver_spec *keyserver)
 
 /* Stubs to avoid linking to photoid.c */
 void
-show_photos (const struct user_attribute *attrs, int count, PKT_public_key *pk)
+show_photos (ctrl_t ctrl, const struct user_attribute *attrs, int count,
+             PKT_public_key *pk, PKT_user_id *uid)
 {
+  (void)ctrl;
   (void)attrs;
   (void)count;
   (void)pk;
+  (void)uid;
 }
 
 int
@@ -733,22 +781,6 @@ agent_get_keyinfo (ctrl_t ctrl, const char *hexkeygrip,
   return gpg_error (GPG_ERR_NO_SECKEY);
 }
 
-gpg_error_t
-gpg_dirmngr_get_pka (ctrl_t ctrl, const char *userid,
-                     unsigned char **r_fpr, size_t *r_fprlen,
-                     char **r_url)
-{
-  (void)ctrl;
-  (void)userid;
-  if (r_fpr)
-    *r_fpr = NULL;
-  if (r_fprlen)
-    *r_fprlen = 0;
-  if (r_url)
-    *r_url = NULL;
-  return gpg_error (GPG_ERR_NOT_FOUND);
-}
-
 gpg_error_t
 export_pubkey_buffer (ctrl_t ctrl, const char *keyspec, unsigned int options,
                       const void *prefix, size_t prefixlen,
@@ -820,6 +852,14 @@ tofu_notice_key_changed (ctrl_t ctrl, kbnode_t kb)
 }
 
 
+const char *
+revocation_reason_code_to_str (int code, char **freeme)
+{
+  (void)code;
+  *freeme = NULL;
+  return "";
+}
+
 int
 get_revocation_reason (PKT_signature *sig, char **r_reason,
                        char **r_comment, size_t *r_commentlen)
@@ -833,3 +873,11 @@ get_revocation_reason (PKT_signature *sig, char **r_reason,
     *r_comment = NULL;
   return 0;
 }
+
+const char *
+impex_filter_getval (void *cookie, const char *propname)
+{
+  (void)cookie;
+  (void)propname;
+  return NULL;
+}
