@@ -29,6 +29,11 @@
 #define ISO7816_CHANGE_REFERENCE_DATA 0x24
 #define ISO7816_RESET_RETRY_COUNTER   0x2C
 
+/* Flags to encode which data should be returned by SELECT.  */
+#define ISO7816_SELECT_NORESP  0  /* no response */
+#define ISO7816_SELECT_FCI     1  /* query FCI (file control information) */
+#define ISO7816_SELECT_FCP     2  /* query FCP (file control parameters)  */
+
 /* Error codes returned by iso7816_verify_status.  A non-negative
  * number gives the number of left tries.
  * NB: The values are also used by the CHV-STATUS lines and thus are
@@ -69,9 +74,14 @@ gpg_error_t iso7816_select_mf (int slot);
 gpg_error_t iso7816_select_file (int slot, int tag, int is_dir);
 gpg_error_t iso7816_select_path (int slot,
                                  const unsigned short *path, size_t pathlen,
-                                 unsigned short top_df);
+                                 unsigned short topdf);
 gpg_error_t iso7816_list_directory (int slot, int list_dirs,
                                     unsigned char **result, size_t *resultlen);
+gpg_error_t iso7816_send_apdu (int slot, int extended_mode,
+                               int class, int ins, int p0, int p1,
+                               int lc, const void *data,
+                               unsigned int *r_sw,
+                               unsigned char **result, size_t *resultlen);
 gpg_error_t iso7816_apdu_direct (int slot,
                                  const void *apdudata, size_t apdudatalen,
                                  int handle_more, unsigned int *r_sw,
@@ -96,6 +106,8 @@ gpg_error_t iso7816_reset_retry_counter_with_rc (int slot, int chvno,
 gpg_error_t iso7816_select_data (int slot, int occurrence, int tag);
 gpg_error_t iso7816_get_data (int slot, int extended_mode, int tag,
                               unsigned char **result, size_t *resultlen);
+gpg_error_t iso7816_get_data_odd (int slot, int extended_mode, unsigned int tag,
+                                  unsigned char **result, size_t *resultlen);
 gpg_error_t iso7816_put_data (int slot, int extended_mode, int tag,
                               const void *data, size_t datalen);
 gpg_error_t iso7816_put_data_odd (int slot, int extended_mode, int tag,
